@@ -5,8 +5,10 @@
 #
 # == param
 # -package Package name.
-# -fields Which fields in DESCRIPTION? Values should be in ``Depends``, ``Imports``, ``LinkingTo``, ``Suggests`` and ``Enhances``.
-# -snapshot If it is ``TRUE``, the package database generated on 2021-10-28 is used. If it is ``FALSE``, the pakage database is directly retrieved from CRAN/Bioconductor.
+# -fields Which fields in DESCRIPTION? Values should be in ``Depends``, ``Imports``, ``LinkingTo``, ``Suggests`` and ``Enhances``. The value can also be
+#     set to ``strong`` or ``weak``.
+# -online Whether use the newest package database directly from CRAN/Bioconductor or the pre-computated package database? 
+#     The version of the pre-computated package database can be set via `pkgndep_opt`$heaviness_db_version.
 #
 # == value
 # A data frame with parent packages as well as their heaviness on ``pacakge``. If ``snapshot`` is set to ``FALSE``, heaviness on child packages
@@ -16,23 +18,23 @@
 # \dontrun{
 # parent_dependency("ComplexHeatmap")
 # }
-parent_dependency = function(package, fields = NULL, snapshot = TRUE) {
+parent_dependency = function(package, fields = NULL, online = FALSE) {
 
-	if(snapshot) {
+	if(!online) {
 		load_all_pkg_dep()
-		lt = env$all_pkg_dep
+		lt = ENV$all_pkg_dep
 	}
 
 	if(inherits(package, "pkgndep")) {
 		package = package$package
 	}
 
-	if(snapshot) {
-		load_pkg_db(snapshot = TRUE)
-		tb = env$pkg_db_snapshot$get_dependency_table(package)
+	if(!online) {
+		load_pkg_db(online = FALSE)
+		tb = ENV$pkg_db_snapshot$get_dependency_table(package)
 	} else {
-		load_pkg_db(snapshot = FALSE)
-		tb = env$pkg$get_dependency_table(package)
+		load_pkg_db(online = TRUE)
+		tb = ENV$pkg_db$get_dependency_table(package)
 	}
 	if(is.null(tb)) {
 		return(data.frame(parents = character(0), children = character(0), dep_fields = character(0), heaviness = numeric(0)))
@@ -42,13 +44,21 @@ parent_dependency = function(package, fields = NULL, snapshot = TRUE) {
 
 	l = rep(TRUE, nrow(tb))
 	if(!is.null(fields)) {
+
+		if("strong" %in% fields) {
+			fields = c(fields, c("Depends", "Imports", "LinkingTo"))
+		}
+		if("weak" %in% fields) {
+			fields = c(fields, c("Suggests", "Enhances"))
+		}
+
 		l = tb[, "dep_fields"] %in% fields
 		tb = tb[l, , drop = FALSE]
 	}
 
 	tb = as.data.frame(tb)
 
-	if(snapshot) {
+	if(!online) {
 		x = lt[[ package ]]
 		if(is.null(x)) {
 			heaviness = rep(0, nrow(x$dep_mat))
@@ -68,8 +78,10 @@ parent_dependency = function(package, fields = NULL, snapshot = TRUE) {
 #
 # == param
 # -package Package name.
-# -fields Which fields in DESCRIPTION? Values should be in ``Depends``, ``Imports``, ``LinkingTo``, ``Suggests`` and ``Enhances``.
-# -snapshot If it is ``TRUE``, the package database generated on 2021-10-28 is used. If it is ``FALSE``, the pakage database is directly retrieved from CRAN/Bioconductor.
+# -fields Which fields in DESCRIPTION? Values should be in ``Depends``, ``Imports``, ``LinkingTo``, ``Suggests`` and ``Enhances``. The value can also be
+#     set to ``strong`` or ``weak``.
+# -online Whether use the newest package database directly from CRAN/Bioconductor or the pre-computated package database? 
+#     The version of the pre-computated package database can be set via `pkgndep_opt`$heaviness_db_version.
 #
 # == value
 # A data frame with child packages as well as its heaviness on its child packages. If ``snapshot`` is set to ``FALSE``, heaviness on child packages
@@ -79,23 +91,23 @@ parent_dependency = function(package, fields = NULL, snapshot = TRUE) {
 # \dontrun{
 # child_dependency("ComplexHeatmap")
 # }
-child_dependency = function(package, fields = NULL, snapshot = TRUE) {
+child_dependency = function(package, fields = NULL, online = FALSE) {
 
-	if(snapshot) {
+	if(!online) {
 		load_all_pkg_dep()
-		lt = env$all_pkg_dep
+		lt = ENV$all_pkg_dep
 	}
 
 	if(inherits(package, "pkgndep")) {
 		package = package$package
 	}
 
-	if(snapshot) {
-		load_pkg_db(snapshot = TRUE)
-		tb = env$pkg_db_snapshot$get_rev_dependency_table(package)
+	if(!online) {
+		load_pkg_db(online = FALSE)
+		tb = ENV$pkg_db_snapshot$get_rev_dependency_table(package)
 	} else {
-		load_pkg_db(snapshot = FALSE)
-		tb = env$pkg_db$get_rev_dependency_table(package)
+		load_pkg_db(online = TRUE)
+		tb = ENV$pkg_db$get_rev_dependency_table(package)
 	}
 	if(is.null(tb)) {
 		return(data.frame(parents = character(0), children = character(0), dep_fields = character(0), heaviness = numeric(0)))
@@ -104,6 +116,13 @@ child_dependency = function(package, fields = NULL, snapshot = TRUE) {
 	colnames(tb) = c("parents", "children", "dep_fields")
 
 	if(!is.null(fields)) {
+
+		if("strong" %in% fields) {
+			fields = c(fields, c("Depends", "Imports", "LinkingTo"))
+		}
+		if("weak" %in% fields) {
+			fields = c(fields, c("Suggests", "Enhances"))
+		}
 		l = tb[, "dep_fields"] %in% fields
 		tb = tb[l, , drop = FALSE]
 	}
@@ -113,7 +132,7 @@ child_dependency = function(package, fields = NULL, snapshot = TRUE) {
 	tb = tb[order(tb$dep_fields, tb$children), , drop = FALSE]
 	tb = tb[!duplicated(tb$children), , drop = FALSE]
 
-	if(snapshot) {
+	if(!online) {
 		heaviness = numeric(nrow(tb))
 		for(i in seq_len(nrow(tb))) {
 			x = lt[[ tb[i, "children"] ]]
@@ -136,7 +155,8 @@ child_dependency = function(package, fields = NULL, snapshot = TRUE) {
 #
 # == param
 # -package Package name.
-# -snapshot If it is ``TRUE``, the package database generated on 2021-10-28 is used. If it is ``FALSE``, the pakage database is directly retrieved from CRAN/Bioconductor.
+# -online Whether use the newest package database directly from CRAN/Bioconductor or the pre-computated package database? 
+#     The version of the pre-computated package database can be set via `pkgndep_opt`$heaviness_db_version.
 #
 # == details
 # Upstream packages with relations of "Depends", "Imports" and "LinkingTo" are retrieved.
@@ -148,9 +168,9 @@ child_dependency = function(package, fields = NULL, snapshot = TRUE) {
 # \dontrun{
 # upstream_dependency("ComplexHeatmap")
 # }
-upstream_dependency = function(package, snapshot = TRUE) {
+upstream_dependency = function(package, online = FALSE) {
 
-	tb = parent_dependency(package, fields = c("Depends", "Imports", "LinkingTo"), snapshot = snapshot)
+	tb = parent_dependency(package, fields = c("Depends", "Imports", "LinkingTo"), online = online)
 
 	if(nrow(tb) == 0) {
 		return(tb)
@@ -161,12 +181,12 @@ upstream_dependency = function(package, snapshot = TRUE) {
 	dep_pkg = tb[, "parents"]
 	while(length(dep_pkg)) {
 		tbl2 = lapply(dep_pkg, function(p) {
-			parent_dependency(p, fields = c("Depends", "Imports", "LinkingTo"), snapshot = snapshot)
+			parent_dependency(p, fields = c("Depends", "Imports", "LinkingTo"), online = online)
 		})
 		tbl2 = tbl2[sapply(tbl2, nrow) > 0]
 
 		tbl = c(tbl, tbl2)
-		dep_pkg = unique(unlist(lapply(tbl2, function(x) x[, "parents"])))
+		dep_pkg = setdiff(unique(unlist(lapply(tbl2, function(x) x[, "parents"]))), unlist(lapply(tbl, function(x) x[, "children"])))
 	}
 	tb = do.call(rbind, tbl)
 
@@ -178,7 +198,8 @@ upstream_dependency = function(package, snapshot = TRUE) {
 #
 # == param
 # -package Package name.
-# -snapshot If it is ``TRUE``, the package database generated on 2021-10-28 is used. If it is ``FALSE``, the pakage database is directly retrieved from CRAN/Bioconductor.
+# -online Whether use the newest package database directly from CRAN/Bioconductor or the pre-computated package database? 
+#     The version of the pre-computated package database can be set via `pkgndep_opt`$heaviness_db_version.
 #
 # == details
 # Downstream packages with relations of ``Depends``, ``Imports`` and ``LinkingTo`` are retrieved.
@@ -190,9 +211,9 @@ upstream_dependency = function(package, snapshot = TRUE) {
 # \dontrun{
 # downstream_dependency("ComplexHeatmap")
 # }
-downstream_dependency = function(package, snapshot = TRUE) {
+downstream_dependency = function(package, online = FALSE) {
 
-	tb = child_dependency(package, fields = c("Depends", "Imports", "LinkingTo"), snapshot = snapshot)
+	tb = child_dependency(package, fields = c("Depends", "Imports", "LinkingTo"), online = online)
 
 	if(nrow(tb) == 0) {
 		return(tb)
@@ -203,12 +224,12 @@ downstream_dependency = function(package, snapshot = TRUE) {
 	children_pkg = tb[, "children"]
 	while(length(children_pkg)) {
 		tbl2 = lapply(children_pkg, function(p) {
-			child_dependency(p, fields = c("Depends", "Imports", "LinkingTo"), snapshot = snapshot)
+			child_dependency(p, fields = c("Depends", "Imports", "LinkingTo"), online = online)
 		})
 		tbl2 = tbl2[sapply(tbl2, nrow) > 0]
 
 		tbl = c(tbl, tbl2)
-		children_pkg = unique(unlist(lapply(tbl2, function(x) x[, "children"])))
+		children_pkg = setdiff(unique(unlist(lapply(tbl2, function(x) x[, "children"]))), unlist(lapply(tbl, function(x) x[, "parents"])))
 	}
 	tb = do.call(rbind, tbl)
 
@@ -239,7 +260,7 @@ is_parent = function(parent, child, ...) {
 }
 
 # == title
-# Test upstream - downstream relationship
+# Test upstream-downstream relationship
 #
 # == param
 # -upstream A vector of package names.
