@@ -1,7 +1,7 @@
 
 
 # == title
-# HTML report for package dependency analysis
+# HTML report for package dependency heaviness analysis
 #
 # == param
 # -pkg An object from `pkgndep`.
@@ -13,9 +13,9 @@
 # == example
 # if(interactive()) {
 #     x = readRDS(system.file("extdata", "ComplexHeatmap_dep.rds", package = "pkgndep"))
-#     dependency_report(x)
+#     heaviness_report(x)
 # }
-dependency_report = function(pkg, file = NULL) {
+heaviness_report = function(pkg, file = NULL) {
 	
 	template_file = system.file("extdata", "package_report.html", package = "pkgndep")
 
@@ -54,3 +54,16 @@ dependency_report = function(pkg, file = NULL) {
 
 	invisible(file)
 }
+
+
+# == title
+# HTML report for package dependency heaviness analysis
+#
+# == param
+# -... Pass to `heaviness_report`.
+#
+# == details
+# It is the same as `heaviness_report`.
+dependency_report = function(...) {
+	heaviness_report(...)
+}
\ No newline at end of file
